/* 
   sharesecret - ShareSecret joins and splits secrets securely.

   Copyright (C) 2000 Stefan Karrmann

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

$Id: sharesecret.h,v 1.3 2000/02/27 21:48:16 sk Exp $
$Log: sharesecret.h,v $
Revision 1.3  2000/02/27 21:48:16  sk
*** empty log message ***

Revision 1.2  2000/02/27 21:37:33  sk
config.h.in generated

Revision 1.1  2000/02/27 20:52:28  sk
forgotten

Revision 1.1  2000/02/13 23:13:29  sk
worked on commandline options

*/

#ifndef SHARESECRET_H
#define  SHARESECRET_H 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <getopt.h>
#include "system.h"
#include "checkopt.h"
#include <gmp.h>


int main (int argc, char **argv);

#ifndef HAVE_MPZ_XOR
void mpz_xor (mpz_t dst, const mpz_t src1, const mpz_t src2);
#endif

#ifdef HAVE_NANA_H
# include <nana.h>
#elif HAVE_ASSERT_H
# include <assert.h>
# define I(x) do { assert(x); } while (0 == 1)
#else
# define I(x) do { 1; } while (0 == 1)
# warning "Cannot check assertions"
#endif HAVE_NANA_H

/* generates a random polynom p of degree min-1 with p(0) = secret
 * and evaluates it such that result[i] = p(i+1)
 * initializes elements x, y and modulus, callee must (mpz_)clear them.
 */
void generate (mpz_t ** x, mpz_t ** y, mpz_t modulus,
		      const mpz_t secret, ulong min, ulong size);
void xor_generate (mpz_t ** result, const mpz_t secret, size_t size);

/* Using the Aitken Neville Algorithem:
 * init: p_j(x) = y[j]
 * recusion equation:
 * p_0...k(x) = 1/(x_k-x_0)*((x-x_k)p_0..k-1(x)-(x_0-x)p_1..k(x))
 *
 * initializes result, callee must (mpz_)clear it.
 * changes and frees y !
 */

void solve (mpz_t result, mpz_t modulus, size_t size,
	    mpz_t * x, mpz_t * y);

/* initializes result, callee must (mpz_)clear it. */
void xor_solve (mpz_t result, size_t size, mpz_t * x);

/* *init* and set r */
void gen_random (mpz_t r, ulong size);

/* *init and set prime */
void generate_prime (mpz_t prime, ulong size);

#if __STDC__
# define VOID void
#else
# define VOID char
#endif

#include <sys/types.h>

#if STDC_HEADERS
# include <stdlib.h>
#else
VOID *calloc ();
VOID *malloc ();
VOID *realloc ();
void free ();
#endif

#if ENABLE_NLS
# include <libintl.h>
# define _(Text) gettext (Text)
#else
# define textdomain(Domain)
# define _(Text) Text
#endif

#include "error.h"

/* Prototypes for functions defined here.  */
#if defined (__STDC__) && __STDC__
static VOID *fixup_null_alloc (size_t n);
VOID *xmalloc (size_t n);
VOID *xcalloc (size_t n, size_t s);
VOID *xrealloc (VOID *p, size_t n);
char *xstrdup (char *p);
#endif

#endif /* SHARESECRET_H */
